// kaldifeat/python/csrc/feature-plp.h
//
// Copyright (c)  2021  Xiaomi Corporation (authors: Fangjun Kuang)

#ifndef KALDIFEAT_PYTHON_CSRC_FEATURE_PLP_H_
#define KALDIFEAT_PYTHON_CSRC_FEATURE_PLP_H_

#include "kaldifeat/python/csrc/kaldifeat.h"

namespace kaldifeat {

void PybindFeaturePlp(py::module &m);

}  // namespace kaldifeat

#endif  // KALDIFEAT_PYTHON_CSRC_FEATURE_PLP_H_
